/*	Copyright (C) 1999.  Intel Corporation.                        */
/*	Copyright (C) 2003-2005. Motorola.                             */

/*
* Sharp LCD term screen,  double panel
* 
* The following code sets up the Sharp LCD panel
* to use a 16x9 fixed font to display 26 rows of 64 chars.
* This routine drives the upper and lower panels.
*/

#include <blob/pxa.h>
#include <blob/lcd_ezx.h>
#include <blob/serial.h>

#define NEW_LCD

#define LCD_X 240
#define LCD_Y 320

/* value = 0x0RGB, so 0x0FFF = white, 0x0000 = black, 0x0F00 = Red */
//short pal_fg = 0x0F0F;      /* 12-bit foreground palette */
//short pal_bg = 0x0FFF;      /* 12-bit background palette */
//short pal_fg = 0x0FFF;       /* white */ /* 12-bit foreground palette */
//short pal_bg = 0x000F;       /* blue  */ /* 12-bit background palette */
//short pal_fg = 0x0000;      /* 12-bit foreground palette */
//short pal_bg = 0x0FFF;      /* 12-bit background palette */
static unsigned short pal_fg = 0;		/* pallet #0 = White */ /* 8-bit foreground palette for passive */
static unsigned short pal_bg = 3;       	/* pallet #3 = Blue  */ /* 8-bit background palette for passive */

static int lcd_row;
static int lcd_col;

struct FrameBuffer_8bit
{
        unsigned short palette [256];
        unsigned char pixel[320][240];
};

struct _LCD_Descriptor
{
        unsigned long FDADR;
        unsigned long FSADR;
        unsigned long FIDR;
        unsigned long LDCMD;
};

struct FrameBuffer_8bit	*fb_8bit;
volatile struct _LCD_Descriptor	*LCD_Descriptor_PAL, *LCD_Descriptor_HI, *LCD_Descriptor_LO;

/*	Copyright (C) 1998.  Intel Corporation.                        */

/*---------------------------------------------------------------------------*/
/* font table for the ADU VGA driver                                         */
/*---------------------------------------------------------------------------*/
static unsigned char font9x16[] =
        {
                /* char code 0x0  */

                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0x1  */

                0x00,0x00,0x7e,0x81,0xa5,0x81,0x81,0xbd,
                0x99,0x81,0x81,0x7e,0x00,0x00,0x00,0x00,

                /* char code 0x2  */

                0x00,0x00,0x7c,0xfe,0xfe,0xd6,0xfe,0xfe,
                0xba,0xc6,0xfe,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x3  */

                0x00,0x00,0x00,0x6c,0xee,0xfe,0xfe,0xfe,
                0xfe,0x7c,0x38,0x10,0x00,0x00,0x00,0x00,

                /* char code 0x4  */

                0x00,0x00,0x00,0x10,0x38,0x7c,0xfe,0x7c,
                0x38,0x10,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0x5  */

                0x00,0x00,0x00,0x10,0x38,0x38,0x10,0x6c,
                0xee,0x6c,0x10,0x38,0x00,0x00,0x00,0x00,

                /* char code 0x6  */

                0x00,0x00,0x10,0x38,0x7c,0x7c,0xfe,0xfe,
                0xfe,0x6c,0x10,0x38,0x00,0x00,0x00,0x00,

                /* char code 0x7  */

                0x00,0x00,0x00,0x00,0x00,0x18,0x3c,0x3c,
                0x3c,0x18,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0x8  */

                0xff,0xff,0xff,0xff,0xff,0xe7,0xc3,0xc3,
                0xc3,0xe7,0xff,0xff,0xff,0xff,0xff,0xff,

                /* char code 0x9  */

                0x00,0x00,0x00,0x00,0x18,0x3c,0x66,0x66,
                0x66,0x3c,0x18,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xa  */

                0xff,0xff,0xff,0xff,0xe7,0xc3,0x99,0x99,
                0x99,0xc3,0xe7,0xff,0xff,0xff,0xff,0xff,

                /* char code 0xb  */

                0x00,0x00,0x1e,0xe,0x1e,0x36,0x78,0xcc,
                0xcc,0xcc,0xcc,0x78,0x00,0x00,0x00,0x00,

                /* char code 0xc  */

                0x00,0x00,0x3c,0x66,0x66,0x66,0x3c,0x18,
                0x7e,0x18,0x18,0x18,0x00,0x00,0x00,0x00,

                /* char code 0xd  */

                0x00,0x00,0x1e,0x1a,0x1e,0x18,0x18,0x18,
                0x18,0x78,0xf8,0x70,0x00,0x00,0x00,0x00,

                /* char code 0xe  */

                0x00,0x00,0x3e,0x36,0x3e,0x36,0x36,0x76,
                0xf6,0x66,0xe,0x1e,0xc,0x00,0x00,0x00,

                /* char code 0xf  */

                0x00,0x00,0x18,0xdb,0x7e,0x3c,0x66,0x66,
                0x3c,0x7e,0xdb,0x18,0x00,0x00,0x00,0x00,

                /* char code 0x10  */

                0x00,0x00,0x00,0x80,0xe0,0xf0,0xfc,0xfe,
                0xfc,0xf0,0xe0,0x80,0x00,0x00,0x00,0x00,

                /* char code 0x11  */

                0x00,0x00,0x00,0x2,0xe,0x3e,0x7e,0xfe,
                0x7e,0x3e,0xe,0x2,0x00,0x00,0x00,0x00,

                /* char code 0x12  */

                0x00,0x00,0x18,0x3c,0x7e,0x18,0x18,0x18,
                0x18,0x7e,0x3c,0x18,0x00,0x00,0x00,0x00,

                /* char code 0x13  */

                0x00,0x00,0x66,0x66,0x66,0x66,0x66,0x66,
                0x66,0x00,0x66,0x66,0x00,0x00,0x00,0x00,

                /* char code 0x14  */

                0x00,0x00,0x7f,0xdb,0xdb,0xdb,0xdb,0x7b,
                0x1b,0x1b,0x1b,0x1b,0x00,0x00,0x00,0x00,

                /* char code 0x15  */

                0x00,0x00,0x7c,0xc6,0xc6,0x60,0x7c,0xf6,
                0xde,0x7c,0xc,0xc6,0xc6,0x7c,0x00,0x00,

                /* char code 0x16  */

                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
                0xfe,0xfe,0xfe,0xfe,0x00,0x00,0x00,0x00,

                /* char code 0x17  */

                0x00,0x00,0x18,0x3c,0x7e,0x18,0x18,0x18,
                0x7e,0x3c,0x18,0x7e,0x00,0x00,0x00,0x00,

                /* char code 0x18  */

                0x00,0x00,0x18,0x3c,0x7e,0x18,0x18,0x18,
                0x18,0x18,0x18,0x18,0x00,0x00,0x00,0x00,

                /* char code 0x19  */

                0x00,0x00,0x18,0x18,0x18,0x18,0x18,0x18,
                0x18,0x7e,0x3c,0x18,0x00,0x00,0x00,0x00,

                /* char code 0x1a  */

                0x00,0x00,0x00,0x00,0x00,0xc,0xe,0xff,
                0xe,0xc,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0x1b  */

                0x00,0x00,0x00,0x00,0x00,0x30,0x70,0xfe,
                0x70,0x30,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0x1c  */

                0x00,0x00,0x00,0x00,0x00,0x00,0xc0,0xc0,
                0xc0,0xfe,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0x1d  */

                0x00,0x00,0x00,0x00,0x00,0x24,0x66,0xff,
                0x66,0x24,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0x1e  */

                0x00,0x00,0x00,0x10,0x38,0x38,0x38,0x7c,
                0x7c,0xfe,0xfe,0x00,0x00,0x00,0x00,0x00,

                /* char code 0x1f  */

                0x00,0x00,0x00,0xfe,0xfe,0x7c,0x7c,0x7c,
                0x38,0x38,0x10,0x00,0x00,0x00,0x00,0x00,

                /* char code 0x20 ' ' */

                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0x21 '!' */

                0x00,0x00,0x18,0x3c,0x3c,0x3c,0x3c,0x18,
                0x18,0x00,0x18,0x18,0x00,0x00,0x00,0x00,

                /* char code 0x22 '"' */

                0x00,0x36,0x36,0x36,0x36,0x14,0x00,0x00,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0x23 '#' */

                0x00,0x00,0x6c,0x6c,0x6c,0xfe,0x6c,0x6c,
                0xfe,0x6c,0x6c,0x6c,0x00,0x00,0x00,0x00,

                /* char code 0x24 '$' */

                0x00,0x00,0x18,0x18,0x7c,0xc6,0xc0,0x78,
                0x3c,0x6,0xc6,0x7c,0x18,0x18,0x00,0x00,

                /* char code 0x25 '%' */

                0x00,0x00,0x00,0x00,0x00,0x62,0x66,0xc,
                0x18,0x30,0x66,0xc6,0x00,0x00,0x00,0x00,

                /* char code 0x26 '&' */

                0x00,0x00,0x38,0x6c,0x38,0x30,0x76,0x7e,
                0xcc,0xcc,0xcc,0x76,0x00,0x00,0x00,0x00,

                /* char code 0x27 ''' */

                0x00,0xc,0xc,0xc,0x18,0x00,0x00,0x00,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0x28 '(' */

                0x00,0x00,0xc,0x18,0x30,0x30,0x30,0x30,
                0x30,0x30,0x18,0xc,0x00,0x00,0x00,0x00,

                /* char code 0x29 ')' */

                0x00,0x00,0x30,0x18,0xc,0xc,0xc,0xc,
                0xc,0xc,0x18,0x30,0x00,0x00,0x00,0x00,

                /* char code 0x2a '*' */

                0x00,0x00,0x00,0x00,0x00,0x6c,0x38,0xfe,
                0x38,0x6c,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0x2b '+' */

                0x00,0x00,0x00,0x00,0x00,0x18,0x18,0x7e,
                0x18,0x18,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0x2c ',' */

                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
                0x00,0xc,0xc,0xc,0x18,0x00,0x00,0x00,

                /* char code 0x2d '-' */

                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfe,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0x2e '.' */

                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
                0x00,0x00,0x18,0x18,0x00,0x00,0x00,0x00,

                /* char code 0x2f '/' */

                0x00,0x00,0x00,0x1,0x3,0x6,0xc,0x18,
                0x30,0x60,0xc0,0x80,0x00,0x00,0x00,0x00,

                /* char code 0x30 '0' */

                0x00,0x00,0x7e,0xc3,0xc7,0xcf,0xdb,0xF3,
                0xe3,0xc3,0xc3,0x7e,0x00,0x00,0x00,0x00,

                /* char code 0x31 '1' */

                0x00,0x00,0x18,0x78,0x18,0x18,0x18,0x18,
                0x18,0x18,0x18,0x7e,0x00,0x00,0x00,0x00,

                /* char code 0x32 '2' */

                0x00,0x00,0x7c,0xc6,0xc6,0x6,0xc,0x18,
                0x30,0x60,0xc6,0xfe,0x00,0x00,0x00,0x00,

                /* char code 0x33 '3' */

                0x00,0x00,0x7c,0xc6,0x6,0x6,0x3c,0x6,
                0x6,0x6,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x34 '4' */

                0x00,0x00,0xc,0x1c,0x3c,0x6c,0xcc,0xcc,
                0xfe,0xc,0xc,0x1e,0x00,0x00,0x00,0x00,

                /* char code 0x35 '5' */

                0x00,0x00,0xfe,0xc0,0xc0,0xc0,0xfc,0x6,
                0x6,0x6,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x36 '6' */

                0x00,0x00,0x7c,0xc6,0xc0,0xc0,0xfc,0xc6,
                0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x37 '7' */

                0x00,0x00,0xfe,0xc6,0x6,0xc,0x18,0x30,
                0x30,0x30,0x30,0x30,0x00,0x00,0x00,0x00,

                /* char code 0x38 '8' */

                0x00,0x00,0x7c,0xc6,0xc6,0xc6,0x7c,0xc6,
                0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x39 '9' */

                0x00,0x00,0x7c,0xc6,0xc6,0xc6,0xc6,0x7e,
                0x6,0x6,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x3a ':' */

                0x00,0x00,0x00,0x00,0x00,0xc,0xc,0x00,
                0x00,0xc,0xc,0x00,0x00,0x00,0x00,0x00,

                /* char code 0x3b ';' */

                0x00,0x00,0x00,0x00,0x00,0xc,0xc,0x00,
                0x00,0xc,0xc,0xc,0x18,0x00,0x00,0x00,

                /* char code 0x3c '<' */

                0x00,0x00,0x00,0xc,0x18,0x30,0x60,0xc0,
                0x60,0x30,0x18,0xc,0x00,0x00,0x00,0x00,

                /* char code 0x3d '=' */

                0x00,0x00,0x00,0x00,0x00,0x00,0xfe,0x00,
                0xfe,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0x3e '>' */

                0x00,0x00,0x00,0x60,0x30,0x18,0xc,0x6,
                0xc,0x18,0x30,0x60,0x00,0x00,0x00,0x00,

                /* char code 0x3f '?' */

                0x00,0x00,0x7c,0xc6,0xc6,0xc,0x18,0x18,
                0x18,0x00,0x18,0x18,0x00,0x00,0x00,0x00,

                /* char code 0x40 '@' */

                0x00,0x00,0x7c,0xc6,0xc6,0xc6,0xde,0xde,
                0xde,0xdc,0xc0,0x7e,0x00,0x00,0x00,0x00,

                /* char code 0x41 'A' */

                0x00,0x00,0x38,0x6c,0xc6,0xc6,0xc6,0xfe,
                0xc6,0xc6,0xc6,0xc6,0x00,0x00,0x00,0x00,

                /* char code 0x42 'B' */

                0x00,0x00,0xfc,0x66,0x66,0x66,0x7c,0x66,
                0x66,0x66,0x66,0xfc,0x00,0x00,0x00,0x00,

                /* char code 0x43 'C' */

                0x00,0x00,0x3c,0x66,0xc2,0xc0,0xc0,0xc0,
                0xc0,0xc2,0x66,0x3c,0x00,0x00,0x00,0x00,

                /* char code 0x44 'D' */

                0x00,0x00,0xf8,0x6c,0x66,0x66,0x66,0x66,
                0x66,0x66,0x6c,0xf8,0x00,0x00,0x00,0x00,

                /* char code 0x45 'E' */

                0x00,0x00,0xfe,0x66,0x60,0x64,0x7c,0x64,
                0x60,0x60,0x66,0xfe,0x00,0x00,0x00,0x00,

                /* char code 0x46 'F' */

                0x00,0x00,0xfe,0x66,0x60,0x64,0x7c,0x64,
                0x60,0x60,0x60,0xf0,0x00,0x00,0x00,0x00,

                /* char code 0x47 'G' */

                0x00,0x00,0x7c,0xc6,0xc6,0xc0,0xc0,0xc0,
                0xce,0xc6,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x48 'H' */

                0x00,0x00,0xc6,0xc6,0xc6,0xc6,0xfe,0xc6,
                0xc6,0xc6,0xc6,0xc6,0x00,0x00,0x00,0x00,

                /* char code 0x49 'I' */

                0x00,0x00,0x3c,0x18,0x18,0x18,0x18,0x18,
                0x18,0x18,0x18,0x3c,0x00,0x00,0x00,0x00,

                /* char code 0x4a 'J' */

                0x00,0x00,0x3c,0x18,0x18,0x18,0x18,0x18,
                0x18,0xd8,0xd8,0x70,0x00,0x00,0x00,0x00,

                /* char code 0x4b 'K' */

                0x00,0x00,0xc6,0xc6,0xcc,0xd8,0xf0,0xf0,
                0xd8,0xcc,0xc6,0xc6,0x00,0x00,0x00,0x00,

                /* char code 0x4c 'L' */

                0x00,0x00,0xf0,0x60,0x60,0x60,0x60,0x60,
                0x60,0x62,0x66,0xfe,0x00,0x00,0x00,0x00,

                /* char code 0x4d 'M' */

                0x00,0x00,0xc3,0xc3,0xe7,0xff,0xff,0xdb,
                0xdb,0xc3,0xc3,0xc3,0x00,0x00,0x00,0x00,

                /* char code 0x4e 'N' */

                0x00,0x00,0xc6,0xc6,0xe6,0xe6,0xf6,0xde,
                0xce,0xce,0xc6,0xc6,0x00,0x00,0x00,0x00,

                /* char code 0x4f 'O' */

                0x00,0x00,0x7c,0xc6,0xc6,0xc6,0xc6,0xc6,
                0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x50 'P' */

                0x00,0x00,0xfc,0x66,0x66,0x66,0x66,0x7c,
                0x60,0x60,0x60,0xf0,0x00,0x00,0x00,0x00,

                /* char code 0x51 'Q' */

                0x00,0x00,0x7c,0xc6,0xc6,0xc6,0xc6,0xc6,
                0xc6,0xd6,0xdE,0x7c,0xE,0x00,0x00,0x00,

                /* char code 0x52 'R' */

                0x00,0x00,0xfc,0x66,0x66,0x66,0x7c,0x78,
                0x6c,0x66,0x66,0xe6,0x00,0x00,0x00,0x00,

                /* char code 0x53 'S' */

                0x00,0x00,0x7c,0xc6,0xc0,0xc0,0x70,0x1c,
                0x6,0x6,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x54 'T' */

                0x00,0x00,0xff,0x99,0x18,0x18,0x18,0x18,
                0x18,0x18,0x18,0x3c,0x00,0x00,0x00,0x00,

                /* char code 0x55 'U' */

                0x00,0x00,0xc6,0xc6,0xc6,0xc6,0xc6,0xc6,
                0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x56 'V' */

                0x00,0x00,0xc3,0xc3,0xc3,0xc3,0xc3,0xc3,
                0xc3,0x66,0x3c,0x18,0x00,0x00,0x00,0x00,

                /* char code 0x57 'W' */

                0x00,0x00,0xc3,0xc3,0xc3,0xc3,0xdb,0xdb,
                0xdb,0xdb,0xff,0x66,0x00,0x00,0x00,0x00,

                /* char code 0x58 'X' */

                0x00,0x00,0xc3,0xc3,0xc3,0x66,0x3c,0x3c,
                0x66,0xc3,0xc3,0xc3,0x00,0x00,0x00,0x00,

                /* char code 0x59 'Y' */

                0x00,0x00,0xc3,0xc3,0xc3,0xc3,0x66,0x3c,
                0x18,0x18,0x18,0x3c,0x00,0x00,0x00,0x00,

                /* char code 0x5a 'Z' */

                0x00,0x00,0xfe,0xc6,0x86,0xc,0x18,0x30,
                0x60,0xc2,0xc6,0xfe,0x00,0x00,0x00,0x00,

                /* char code 0x5b '[' */

                0x00,0x00,0x7c,0x60,0x60,0x60,0x60,0x60,
                0x60,0x60,0x60,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x5c '\' */

                0x00,0x00,0x00,0x80,0xc0,0x60,0x30,0x18,
                0xc,0x6,0x3,0x1,0x00,0x00,0x00,0x00,

                /* char code 0x5d ']' */

                0x00,0x00,0x7c,0xc,0xc,0xc,0xc,0xc,
                0xc,0xc,0xc,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x5e '^' */

                0x00,0x10,0x38,0x6c,0xc6,0x00,0x00,0x00,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0x5f '_' */

                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
                0x00,0x00,0x00,0x00,0x00,0xff,0x00,0x00,

                /* char code 0x60 '`' */

                0x00,0x18,0x18,0x18,0xc,0x00,0x00,0x00,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0x61 'a' */

                0x00,0x00,0x00,0x00,0x00,0x78,0xc,0x7c,
                0xcc,0xcc,0xdc,0x76,0x00,0x00,0x00,0x00,

                /* char code 0x62 'b' */

                0x00,0x00,0xe0,0x60,0x60,0x7c,0x66,0x66,
                0x66,0x66,0x66,0xfc,0x00,0x00,0x00,0x00,

                /* char code 0x63 'c' */

                0x00,0x00,0x00,0x00,0x00,0x7c,0xc6,0xc0,
                0xc0,0xc0,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x64 'd' */

                0x00,0x00,0x1c,0xc,0xc,0x7c,0xcc,0xcc,
                0xcc,0xcc,0xcc,0x7e,0x00,0x00,0x00,0x00,

                /* char code 0x65 'e' */

                0x00,0x00,0x00,0x00,0x00,0x7c,0xc6,0xcc,
                0xd8,0xf0,0xe6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x66 'f' */

                0x00,0x00,0x1c,0x36,0x30,0x30,0xfc,0x30,
                0x30,0x30,0x30,0x78,0x00,0x00,0x00,0x00,

                /* char code 0x67 'g' */

                0x00,0x00,0x00,0x00,0x00,0x76,0xce,0xc6,
                0xc6,0xce,0x76,0x6,0xc6,0x7c,0x00,0x00,

                /* char code 0x68 'h' */

                0x00,0x00,0xe0,0x60,0x60,0x7c,0x66,0x66,
                0x66,0x66,0x66,0xe6,0x00,0x00,0x00,0x00,

                /* char code 0x69 'i' */

                0x00,0x00,0x18,0x10,0x00,0x38,0x18,0x18,
                0x18,0x18,0x18,0x3c,0x00,0x00,0x00,0x00,

                /* char code 0x6a 'j' */

                0x00,0x00,0xc,0x8,0x00,0x1c,0xc,0xc,
                0xc,0xc,0xc,0xcc,0xcc,0x78,0x00,0x00,

                /* char code 0x6b 'k' */

                0x00,0x00,0xe0,0x60,0x60,0x66,0x66,0x6c,
                0x78,0x6c,0x66,0xe6,0x00,0x00,0x00,0x00,

                /* char code 0x6c 'l' */

                0x00,0x00,0x18,0x18,0x18,0x18,0x18,0x18,
                0x18,0x18,0x18,0x1c,0x00,0x00,0x00,0x00,

                /* char code 0x6d 'm' */

                0x00,0x00,0x00,0x00,0x00,0x66,0xff,0xdb,
                0xdb,0xdb,0xc3,0xc3,0x00,0x00,0x00,0x00,

                /* char code 0x6e 'n' */

                0x00,0x00,0x00,0x00,0x00,0xdc,0x66,0x66,
                0x66,0x66,0x66,0x66,0x00,0x00,0x00,0x00,

                /* char code 0x6f 'o' */

                0x00,0x00,0x00,0x00,0x00,0x7c,0xc6,0xc6,
                0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x70 'p' */

                0x00,0x00,0x00,0x00,0x00,0xdc,0x66,0x66,
                0x66,0x66,0x7c,0x60,0x60,0xf0,0x00,0x00,

                /* char code 0x71 'q' */

                0x00,0x00,0x00,0x00,0x00,0x76,0xcc,0xcc,
                0xcc,0xcc,0x7c,0xc,0xc,0x1e,0x00,0x00,

                /* char code 0x72 'r' */

                0x00,0x00,0x00,0x00,0x00,0xdc,0x66,0x60,
                0x60,0x60,0x60,0xf0,0x00,0x00,0x00,0x00,

                /* char code 0x73 's' */

                0x00,0x00,0x00,0x00,0x00,0x7c,0xc6,0xc0,
                0x7c,0x6,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x74 't' */

                0x00,0x00,0x30,0x30,0x30,0xfc,0x30,0x30,
                0x30,0x30,0x36,0x1c,0x00,0x00,0x00,0x00,

                /* char code 0x75 'u' */

                0x00,0x00,0x00,0x00,0x00,0xcc,0xcc,0xcc,
                0xcc,0xcc,0xcc,0x76,0x00,0x00,0x00,0x00,

                /* char code 0x76 'v' */

                0x00,0x00,0x00,0x00,0x00,0xc3,0xc3,0xc3,
                0xc3,0x66,0x3c,0x18,0x00,0x00,0x00,0x00,

                /* char code 0x77 'w' */

                0x00,0x00,0x00,0x00,0x00,0xc3,0xc3,0xdb,
                0xdb,0xdb,0xff,0x66,0x00,0x00,0x00,0x00,

                /* char code 0x78 'x' */

                0x00,0x00,0x00,0x00,0x00,0xc6,0xc6,0x6c,
                0x38,0x6c,0xc6,0xc6,0x00,0x00,0x00,0x00,

                /* char code 0x79 'y' */

                0x00,0x00,0x00,0x00,0x00,0xc6,0xc6,0xc6,
                0xc6,0xce,0x76,0x6,0xc6,0x7c,0x00,0x00,

                /* char code 0x7a 'z' */

                0x00,0x00,0x00,0x00,0x00,0xfe,0x86,0xc,
                0x18,0x30,0x62,0xfe,0x00,0x00,0x00,0x00,

                /* char code 0x7b '{' */

                0x00,0x00,0xe,0x18,0x18,0x18,0x70,0x18,
                0x18,0x18,0x18,0xe,0x00,0x00,0x00,0x00,

                /* char code 0x7c '|' */

                0x00,0x00,0x18,0x18,0x18,0x18,0x00,0x18,
                0x18,0x18,0x18,0x18,0x00,0x00,0x00,0x00,

                /* char code 0x7d '}' */

                0x00,0x00,0x70,0x18,0x18,0x18,0xe,0x18,
                0x18,0x18,0x18,0x70,0x00,0x00,0x00,0x00,

                /* char code 0x7e '~' */

                0x00,0x00,0x76,0xdc,0x00,0x00,0x00,0x00,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0x7f  */

                0x00,0x00,0x00,0x00,0x00,0x10,0x38,0x38,
                0x6c,0x6c,0xfe,0x00,0x00,0x00,0x00,0x00,

                /* char code 0x80  */

                0x00,0x00,0x3c,0x66,0xc0,0xc0,0xc0,0xc6,
                0x66,0x3c,0x18,0xc,0xcc,0x38,0x00,0x00,

                /* char code 0x81  */

                0x00,0x00,0xc6,0x00,0x00,0xc6,0xc6,0xc6,
                0xc6,0xc6,0xce,0x76,0x00,0x00,0x00,0x00,

                /* char code 0x82  */

                0x00,0xc,0x18,0x30,0x00,0x7c,0xc6,0xc6,
                0xfe,0xc0,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x83  */

                0x00,0x30,0x78,0xcc,0x00,0x78,0xc,0x7c,
                0xcc,0xcc,0xdc,0x76,0x00,0x00,0x00,0x00,

                /* char code 0x84  */

                0x00,0x00,0xcc,0x00,0x00,0x78,0xc,0x7c,
                0xcc,0xcc,0xdc,0x76,0x00,0x00,0x00,0x00,

                /* char code 0x85  */

                0x00,0x60,0x30,0x18,0x00,0x78,0xc,0x7c,
                0xcc,0xcc,0xdc,0x76,0x00,0x00,0x00,0x00,

                /* char code 0x86  */

                0x00,0x38,0x6c,0x38,0x00,0x78,0xc,0x7c,
                0xcc,0xcc,0xdc,0x76,0x00,0x00,0x00,0x00,

                /* char code 0x87  */

                0x00,0x00,0x00,0x00,0x7c,0xc6,0xc0,0xc0,
                0xc6,0x7c,0x18,0xc,0x6c,0x38,0x00,0x00,

                /* char code 0x88  */

                0x00,0x30,0x78,0xcc,0x00,0x7c,0xc6,0xc6,
                0xfe,0xc0,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x89  */

                0x00,0x00,0xcc,0x00,0x00,0x7c,0xc6,0xc6,
                0xfe,0xc0,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x8a  */

                0x00,0x30,0x18,0xc,0x00,0x7c,0xc6,0xc6,
                0xfe,0xc0,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x8b  */

                0x00,0x00,0x66,0x00,0x00,0x38,0x18,0x18,
                0x18,0x18,0x18,0x3c,0x00,0x00,0x00,0x00,

                /* char code 0x8c  */

                0x00,0x18,0x3c,0x66,0x00,0x38,0x18,0x18,
                0x18,0x18,0x18,0x3c,0x00,0x00,0x00,0x00,

                /* char code 0x8d  */

                0x00,0x60,0x30,0x18,0x00,0x38,0x18,0x18,
                0x18,0x18,0x18,0x3c,0x00,0x00,0x00,0x00,

                /* char code 0x8e  */

                0x00,0xc6,0x00,0x38,0x6c,0xc6,0xc6,0xc6,
                0xfe,0xc6,0xc6,0xc6,0x00,0x00,0x00,0x00,

                /* char code 0x8f  */

                0x38,0x6c,0x38,0x00,0x38,0x6c,0xc6,0xc6,
                0xfe,0xc6,0xc6,0xc6,0x00,0x00,0x00,0x00,

                /* char code 0x90  */

                0xc,0x18,0x30,0x00,0xfe,0x60,0x60,0x7c,
                0x60,0x60,0x60,0xfe,0x00,0x00,0x00,0x00,

                /* char code 0x91  */

                0x00,0x00,0x00,0x00,0x66,0xdb,0x1b,0x7f,
                0xd8,0xd8,0xdf,0x76,0x00,0x00,0x00,0x00,

                /* char code 0x92  */

                0x00,0x00,0x7e,0xd8,0xd8,0xd8,0xd8,0xfe,
                0xd8,0xd8,0xd8,0xde,0x00,0x00,0x00,0x00,

                /* char code 0x93  */

                0x00,0x30,0x78,0xcc,0x00,0x7c,0xc6,0xc6,
                0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x94  */

                0x00,0x00,0xc6,0x00,0x00,0x7c,0xc6,0xc6,
                0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x95  */

                0x00,0x30,0x18,0xc,0x00,0x7c,0xc6,0xc6,
                0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x96  */

                0x00,0x30,0x78,0xcc,0x00,0xc6,0xc6,0xc6,
                0xc6,0xc6,0xce,0x76,0x00,0x00,0x00,0x00,

                /* char code 0x97  */

                0x00,0x60,0x30,0x18,0x00,0xc6,0xc6,0xc6,
                0xc6,0xc6,0xce,0x76,0x00,0x00,0x00,0x00,

                /* char code 0x98  */

                0x00,0xc6,0x00,0x00,0xc6,0xc6,0xc6,0xc6,
                0xce,0x76,0x6,0x6,0xc6,0x7c,0x00,0x00,

                /* char code 0x99  */

                0x00,0xc6,0x00,0x7c,0xc6,0xc6,0xc6,0xc6,
                0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x9a  */

                0x00,0xc6,0x00,0xc6,0xc6,0xc6,0xc6,0xc6,
                0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0x9b  */

                0x00,0x00,0x18,0x18,0x7c,0xc6,0xc0,0xc0,
                0xc6,0x7c,0x18,0x18,0x00,0x00,0x00,0x00,

                /* char code 0x9c  */

                0x00,0x38,0x6c,0x60,0x60,0xf0,0x60,0x60,
                0x60,0x66,0xf6,0x6c,0x00,0x00,0x00,0x00,

                /* char code 0x9d  */

                0x00,0x66,0x66,0x66,0x66,0x3c,0x18,0x7e,
                0x18,0x3c,0x18,0x18,0x00,0x00,0x00,0x00,

                /* char code 0x9e  */

                0x00,0xfc,0xc6,0xc6,0xfc,0xc0,0xcc,0xde,
                0xcc,0xcc,0xcc,0xc6,0x00,0x00,0x00,0x00,

                /* char code 0x9f  */

                0x00,0xe,0x1b,0x18,0x18,0x18,0x7e,0x18,
                0x18,0x18,0x18,0x18,0xd8,0x70,0x00,0x00,

                /* char code 0xa0  */

                0x00,0xc,0x18,0x30,0x00,0x78,0xc,0x7c,
                0xcc,0xcc,0xdc,0x76,0x00,0x00,0x00,0x00,

                /* char code 0xa1  */

                0x00,0xc,0x18,0x30,0x00,0x38,0x18,0x18,
                0x18,0x18,0x18,0x3c,0x00,0x00,0x00,0x00,

                /* char code 0xa2  */

                0x00,0xc,0x18,0x30,0x00,0x7c,0xc6,0xc6,
                0xc6,0xc6,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0xa3  */

                0x00,0x18,0x30,0x60,0x00,0xcc,0xcc,0xcc,
                0xcc,0xcc,0xdc,0x76,0x00,0x00,0x00,0x00,

                /* char code 0xa4  */

                0x00,0x00,0x76,0xdc,0x00,0xbc,0x66,0x66,
                0x66,0x66,0x66,0xe6,0x00,0x00,0x00,0x00,

                /* char code 0xa5  */

                0x00,0x76,0xdc,0x00,0xc6,0xc6,0xe6,0xf6,
                0xde,0xce,0xc6,0xc6,0x00,0x00,0x00,0x00,

                /* char code 0xa6  */

                0x00,0x3c,0x6c,0x6c,0x3e,0x00,0x7e,0x00,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xa7  */

                0x00,0x38,0x6c,0x6c,0x38,0x00,0x7c,0x00,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xa8  */

                0x00,0x00,0x30,0x30,0x00,0x30,0x30,0x30,
                0x60,0xc6,0xc6,0x7c,0x00,0x00,0x00,0x00,

                /* char code 0xa9  */

                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7e,
                0x60,0x60,0x60,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xaa  */

                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7e,
                0x6,0x6,0x6,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xab  */

                0x00,0x60,0x60,0x62,0x66,0x6c,0x18,0x30,
                0x60,0xdc,0x36,0xc,0x18,0x3e,0x00,0x00,

                /* char code 0xac  */

                0x00,0x60,0x60,0x62,0x66,0x6c,0x18,0x36,
                0x6e,0xde,0x36,0x7e,0x6,0x6,0x00,0x00,

                /* char code 0xad  */

                0x00,0x00,0x18,0x18,0x00,0x18,0x18,0x3c,
                0x3c,0x3c,0x3c,0x18,0x00,0x00,0x00,0x00,

                /* char code 0xae  */

                0x00,0x00,0x00,0x00,0x00,0x36,0x6c,0xd8,
                0x6c,0x36,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xaf  */

                0x00,0x00,0x00,0x00,0x00,0xd8,0x6c,0x36,
                0x6c,0xd8,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xb0  */
                0x11,0x44,0x11,0x44,0x11,0x44,0x11,0x44,
                0x11,0x44,0x11,0x44,0x11,0x44,0x11,0x44,

                /* char code 0xb1  */

                0xaa,0x55,0xaa,0x55,0xaa,0x55,0xaa,0x55,
                0xaa,0x55,0xaa,0x55,0xaa,0x55,0xaa,0x55,

                /* char code 0xb2  */

                0xdd,0x77,0xdd,0x77,0xdd,0x77,0xdd,0x77,
                0xdd,0x77,0xdd,0x77,0xdd,0x77,0xdd,0x77,

                /* char code 0xb3  */

                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,
                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,


                /* char code 0xb4  */

                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0xf8,
                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,

                /* char code 0xb5  */

                0x18,0x18,0x18,0x18,0x18,0xf8,0x18,0xf8,
                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,

                /* char code 0xb6  */

                0x36,0x36,0x36,0x36,0x36,0x36,0x36,0xf6,
                0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,

                /* char code 0xb7  */

                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xfe,
                0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,

                /* char code 0xb8  */

                0x00,0x00,0x00,0x00,0x00,0xf8,0x18,0xf8,
                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,

                /* char code 0xb9  */

                0x36,0x36,0x36,0x36,0x36,0xf6,0x6,0xf6,
                0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,

                /* char code 0xba  */

                0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,
                0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,


                /* char code 0xbb  */

                0x00,0x00,0x00,0x00,0x00,0xfe,0x6,0xf6,
                0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,

                /* char code 0xbc  */

                0x36,0x36,0x36,0x36,0x36,0xf6,0x6,0xfe,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xbd  */

                0x36,0x36,0x36,0x36,0x36,0x36,0x36,0xfe,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,


                /* char code 0xbe  */

                0x18,0x18,0x18,0x18,0x18,0xf8,0x18,0xf8,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xbf  */

                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xf8,
                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,


                /* char code 0xc0  */

                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x1f,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xc1  */

                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0xff,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xc2  */

                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,

                /* char code 0xc3  */

                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x1f,
                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,

                /* char code 0xc4  */

                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xc5  */

                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0xff,
                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,

                /* char code 0xc6  */

                0x18,0x18,0x18,0x18,0x18,0x1f,0x18,0x1f,
                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,

                /* char code 0xc7  */

                0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x37,
                0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,

                /* char code 0xc8  */

                0x36,0x36,0x36,0x36,0x36,0x37,0x30,0x3f,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xc9  */

                0x00,0x00,0x00,0x00,0x00,0x3f,0x30,0x37,
                0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,

                /* char code 0xca  */

                0x36,0x36,0x36,0x36,0x36,0xf7,0x00,0xff,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xcb  */

                0x00,0x00,0x00,0x00,0x00,0xff,0x00,0xf7,
                0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,

                /* char code 0xcc  */

                0x36,0x36,0x36,0x36,0x36,0x37,0x30,0x37,
                0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,

                /* char code 0xcd  */

                0x00,0x00,0x00,0x00,0x00,0xff,0x00,0xff,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xce  */

                0x36,0x36,0x36,0x36,0x36,0xf7,0x00,0xf7,
                0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,


                /* char code 0xcf  */

                0x18,0x18,0x18,0x18,0x18,0xff,0x00,0xff,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xd0  */

                0x36,0x36,0x36,0x36,0x36,0x36,0x36,0xff,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xd1  */

                0x00,0x00,0x00,0x00,0x00,0xff,0x00,0xff,
                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,

                /* char code 0xd2  */

                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xff,
                0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,

                /* char code 0xd3  */

                0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x3f,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xd4  */

                0x18,0x18,0x18,0x18,0x18,0x1f,0x18,0x1f,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xd5  */

                0x00,0x00,0x00,0x00,0x00,0x1f,0x18,0x1f,
                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,

                /* char code 0xd6  */

                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3f,
                0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,

                /* char code 0xd7  */

                0x36,0x36,0x36,0x36,0x36,0x36,0x36,0xff,
                0x36,0x36,0x36,0x36,0x36,0x36,0x36,0x36,

                /* char code 0xd8  */

                0x18,0x18,0x18,0x18,0x18,0xff,0x18,0xff,
                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,

                /* char code 0xd9  */

                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0xf8,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xda  */

                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x1f,
                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,

                /* char code 0xdb  */

                0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
                0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,

                /* char code 0xdc  */

                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
                0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,

                /* char code 0xdd  */

                0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,
                0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,0xf0,

                /* char code 0xde  */

                0xf,0xf,0xf,0xf,0xf,0xf,0xf,0xf,
                0xf,0xf,0xf,0xf,0xf,0xf,0xf,0xf,

                /* char code 0xdf  */

                0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xe0  */

                0x00,0x00,0x00,0x00,0x76,0xdc,0xd8,0xd8,
                0xd8,0xd8,0xdc,0x76,0x00,0x00,0x00,0x00,

                /* char code 0xe1  */

                0x00,0x00,0x00,0x00,0x78,0xcc,0xd8,0xfc,
                0xc6,0xc6,0xe6,0xdc,0xc0,0xc0,0x00,0x00,

                /* char code 0xe2  */

                0x00,0x00,0xfe,0x66,0x62,0x60,0x60,0x60,
                0x60,0x60,0x60,0x60,0x00,0x00,0x00,0x00,

                /* char code 0xe3  */

                0x00,0x00,0x00,0x00,0x00,0xfe,0x6c,0x6c,
                0x6c,0x6c,0x6c,0x6c,0x00,0x00,0x00,0x00,

                /* char code 0xe4  */

                0x00,0x00,0xfe,0xc6,0x62,0x30,0x18,0x18,
                0x30,0x62,0xc6,0xfe,0x00,0x00,0x00,0x00,

                /* char code 0xe5  */

                0x00,0x00,0x00,0x00,0x00,0x7e,0xd8,0xcc,
                0xcc,0xcc,0xd8,0x70,0x00,0x00,0x00,0x00,

                /* char code 0xe6  */

                0x00,0x00,0x00,0x00,0x66,0x66,0x66,0x66,
                0x66,0x7c,0x60,0xc0,0x80,0x00,0x00,0x00,

                /* char code 0xe7  */

                0x00,0x00,0x00,0x00,0x00,0x76,0xdc,0x18,
                0x18,0x18,0x18,0x18,0x00,0x00,0x00,0x00,

                /* char code 0xe8  */

                0x00,0x00,0xfe,0x38,0x38,0x6c,0xc6,0xc6,
                0x6c,0x38,0x38,0xfe,0x00,0x00,0x00,0x00,

                /* char code 0xe9  */

                0x00,0x00,0x00,0x38,0x6c,0xc6,0xc6,0xfe,
                0xc6,0xc6,0x6c,0x38,0x00,0x00,0x00,0x00,

                /* char code 0xea  */

                0x00,0x00,0x38,0x6c,0xc6,0xc6,0xc6,0xc6,
                0x6c,0x6c,0x6c,0xee,0x00,0x00,0x00,0x00,

                /* char code 0xeb  */

                0x00,0x00,0x3e,0x60,0x60,0x3c,0x66,0xc6,
                0xc6,0xc6,0xcc,0x78,0x00,0x00,0x00,0x00,


                /* char code 0xec  */

                0x00,0x00,0x00,0x00,0x00,0x7e,0xdb,0xdb,
                0xdb,0x7e,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xed  */

                0x00,0x00,0x2,0x6,0x7c,0xce,0xde,0xf6,
                0xf6,0x7c,0x60,0xc0,0x00,0x00,0x00,0x00,


                /* char code 0xee  */

                0x00,0x00,0x00,0x1c,0x30,0x60,0x60,0x7c,
                0x60,0x60,0x30,0x1c,0x00,0x00,0x00,0x00,

                /* char code 0xef  */

                0x00,0x00,0x7c,0xc6,0xc6,0xc6,0xc6,0xc6,
                0xc6,0xc6,0xc6,0xc6,0x00,0x00,0x00,0x00,

                /* char code 0xf0  */

                0x00,0x00,0x00,0x00,0xfe,0x00,0x00,0xfe,
                0x00,0x00,0xfe,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xf1  */

                0x00,0x00,0x00,0x00,0x18,0x18,0x7e,0x18,
                0x18,0x00,0x00,0x7e,0x00,0x00,0x00,0x00,

                /* char code 0xf2  */

                0x00,0x00,0x30,0x18,0xc,0x6,0xc,0x18,
                0x30,0x00,0x00,0x7e,0x00,0x00,0x00,0x00,

                /* char code 0xf3  */

                0x00,0x00,0xc,0x18,0x30,0x60,0x30,0x18,
                0xc,0x00,0x00,0x7e,0x00,0x00,0x00,0x00,

                /* char code 0xf4  */

                0x00,0x00,0x00,0x00,0xc,0x1e,0x1a,0x18,
                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,

                /* char code 0xf5  */

                0x18,0x18,0x18,0x18,0x18,0x18,0x18,0x18,
                0x18,0x18,0x58,0x78,0x30,0x00,0x00,0x00,

                /* char code 0xf6  */

                0x00,0x00,0x00,0x00,0x18,0x18,0x00,0x7e,
                0x00,0x18,0x18,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xf7  */

                0x00,0x00,0x00,0x00,0x00,0x00,0x76,0xdc,
                0x00,0x76,0xdc,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xf8  */

                0x00,0x00,0x78,0xcc,0xcc,0x78,0x00,0x00,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xf9  */

                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x18,
                0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xfa  */

                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
                0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xfb  */

                0x00,0x00,0x1f,0x18,0x18,0x18,0x18,0x18,
                0xd8,0xd8,0x78,0x38,0x18,0x00,0x00,0x00,

                /* char code 0xfc  */

                0x00,0x00,0xd8,0x6c,0x6c,0x6c,0x6c,0x6c,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xfd  */

                0x00,0x00,0x70,0xd8,0x18,0x30,0x60,0xf8,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xfe  */

                0x00,0x00,0x00,0x00,0x00,0x7e,0x7e,0x7e,
                0x7e,0x7e,0x7e,0x00,0x00,0x00,0x00,0x00,

                /* char code 0xff  */

                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
                0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00
        };

/*
void cacheflush( void )
{
	int i;
	for( i = 0; i < 32768/4; i++ ) {
		datacacheflush1[i] = datacacheflush2[i];
	}
}
*/
/*****************************************************************************/

/*
* fast screen cleaner
*/
static void clrScreen()
{
        int i;
        unsigned char *p;

        p = &fb_8bit->pixel[0][0]; /* get pointer to start of frame buffer */
        /* clear all of buffer */
        for (i=0; i<(LCD_X*LCD_Y); i++) {
                *p++ = pal_bg;
        };
        lcd_row = 0;
        lcd_col = 0;
}
/*****************************************************************************/



/*
* fast char row cleaner
*/
void clrCharRow(int row)
{

        int i;
        unsigned char *p;

        p = &fb_8bit->pixel[0][0] + (row * 18 * LCD_X) ; /* get pointer to start of row in frame buffer */

        /* clear a row of char in frame buffer */
        for (i=0; i<(((LCD_X)*18)); i++) {
                *p++ = pal_bg;
        };
}
/*****************************************************************************/




/*
* Fast scroll up one line
*/
void scroll_line()
{

        int *f,*t,i;

        f = (int *) (&fb_8bit->pixel[0][0] + (1 * 18 * LCD_X)); /* get pointer to start of row 1 in frame buffer */
        t = (int *) (&fb_8bit->pixel[0][0] + (0 * 18 * LCD_X)); /* get pointer to start of row 0 in frame buffer */

        /* move all rows up one */
        for (i=((LCD_X*18*1)/4); i<((LCD_X*18*16)/4); i++) {
                *t++ = *f++;
        };
        /* clear new row */
        clrCharRow(15);
}
/*****************************************************************************/

void EnableLCD_8bit_active()
{

        unsigned long FrameBufferStart;
        unsigned long FrameBufferLength;

        FrameBufferStart  = SDRAM_B0;		// put framebuffer here (random place in memory)
        FrameBufferLength = LCD_X * LCD_Y;		// LCD panel is 320 x 240 dual panel

        // Disable the LCD for now.
        GPDR3 |=  0x00100000;
        GPSR3  =  0x00100000;
        Delay(40000);
        LCCR0 = 0;

        /*enable LCD panel*/

        /*  //Configure PWM_0 for LCD backlight */
        /*  	PWM_CTRL0   = 0x10; //Prescale of 16 */
        /*  	PWM_PWDUTY0   = 20; */
        /*  	PWM_PERVAL0 = 23; */

        //a17400 from zhou qiong, change bk light
        PWM_CTRL0 = 2;
        PWM_PERVAL0 = 49;
        PWM_PWDUTY0 = 25;

        GPDR0 |=  0x00010000;  //PWM0 is GPIO16
        GAFR0_U |= 0x00000002;  //PWM0 is AF_2 for GP16

        CKEN |= CKEN16_LCD;
        fb_8bit = (struct FrameBuffer_8bit *) (FrameBufferStart);
        // put the GPIOs in the correct direction (0=in, 1=out)
        GPDR1 |= 0xFC000000;		// GPIO 63-58 = 1, (0=in, 1=out)
	GPDR2 |= 0x00003FFF;		// GPIO 77-64 = 1, (0=in, 1=out)

        GAFR1_U = (GAFR1_U & 0x000FFFFF) | 0xAAA00000;	// bit 31,30 = LDD5, bit 21,20 = LDD0
        GAFR2_L = (GAFR2_L & 0xF0000000) | 0x0AAAAAAA;	// bit 27,26 = LBIAS, bit 1,0 = LDD6

        /* remember 5 bits Red, 6 bits Green, 5 bits Blue			*/
        /* make palette 0 -> white, 1..15 -> other colors 		*/
        fb_8bit->palette[0] = 0xFFFF;		// White
        fb_8bit->palette[1] = 0xF800;		// Red
        fb_8bit->palette[2] = 0x07E0;		// Green
        fb_8bit->palette[3] = 0x001F;		// Blue
        fb_8bit->palette[4] = 0x0FF4;
        fb_8bit->palette[5] = 0x04FF;
        fb_8bit->palette[6] = 0x0F4F;
        fb_8bit->palette[7] = 0x0AAA;
        fb_8bit->palette[8] = 0x0444;
        fb_8bit->palette[9] = 0x048C;
        fb_8bit->palette[10] = 0x084C;
        fb_8bit->palette[11] = 0x08C4;
        fb_8bit->palette[12] = 0x0822;
        fb_8bit->palette[13] = 0x0282;
        fb_8bit->palette[14] = 0x0228;
        fb_8bit->palette[15] = 0x0882;
        fb_8bit->palette[16] = 0x0288;
        fb_8bit->palette[17] = 0x0828;


        /* clear screen */
        clrScreen();

        //frame descriptors
        LCD_Descriptor_PAL = (struct _LCD_Descriptor *) (FrameBufferStart + FrameBufferLength + 0x200);	// random spot in SDRAM (16-byte boundary)
        LCD_Descriptor_HI  = (struct _LCD_Descriptor *) (FrameBufferStart + FrameBufferLength + 0x210);	// random spot in SDRAM (16-byte boundary)

        LCD_Descriptor_PAL->FDADR = (unsigned long)LCD_Descriptor_HI;		//point descriptor to upper frame descriptor
        LCD_Descriptor_PAL->FSADR = (unsigned long)&fb_8bit->palette[0];		//point to palette ram
        LCD_Descriptor_PAL->FIDR  = 0;
        LCD_Descriptor_PAL->LDCMD = 0x04000200;						//PAL = 1, Len=512 bytes

        LCD_Descriptor_HI->FDADR = (unsigned long)LCD_Descriptor_HI;			//point descriptor back to itself
        LCD_Descriptor_HI->FSADR = (unsigned long)&fb_8bit->pixel[0][0];		//point to upper half of panel
        LCD_Descriptor_HI->FIDR  = 0;
        LCD_Descriptor_HI->LDCMD = FrameBufferLength;				//Len= 1/2 panel = (640*240)

        //program LCD registers, except FDADRx and LCCR0

#ifdef NEW_LCD

        LCCR1 =		(19 << LCD1_V_BLW) |
                 ( 9 << LCD1_V_ELW) |
                 ( 9 << LCD1_V_HSW) |
                 ((LCD_X-1) << LCD1_V_PPL);


        LCCR2 =		(1 << LCD2_V_BFW) |
                 (1 << LCD2_V_EFW) |
                 (1 << LCD2_V_VSW) |
                 ((LCD_Y-1) << LCD2_V_LPP);


        LCCR3 =		(0    << LCD3_V_DPC) |
                 (3    << LCD3_V_BPP) |
                 (0    << LCD3_V_OEP) |
                 (0    << LCD3_V_PCP) | 			// opposite of SA1110
                 (1    << LCD3_V_HSP) |
                 (1    << LCD3_V_VSP) |
                 (0    << LCD3_V_API) |
                 (0xFF << LCD3_V_ACB) |
                 // (5   << LCD3_V_PCD);			// depends on mem clock!!!
                 (7   << LCD3_V_PCD);			// depends on mem clock!!!
#else

        LCCR1 =		(16 << LCD1_V_BLW) |
                 ( 8 << LCD1_V_ELW) |
                 ( 8 << LCD1_V_HSW) |
                 ((LCD_X-1) << LCD1_V_PPL);


        LCCR2 =		(10 << LCD2_V_BFW) |
                 (10 << LCD2_V_EFW) |
                 (4 << LCD2_V_VSW) |
                 ((LCD_Y-1) << LCD2_V_LPP);


        LCCR3 =		(0    << LCD3_V_DPC) |
                 (3    << LCD3_V_BPP) |
                 (0    << LCD3_V_OEP) |
                 (1    << LCD3_V_PCP) | 			// opposite of SA1110
                 (1    << LCD3_V_HSP) |
                 (0    << LCD3_V_VSP) |
                 (0    << LCD3_V_API) |
                 (0xFF << LCD3_V_ACB) |
                 // (5   << LCD3_V_PCD);			// depends on mem clock!!!
                 (7   << LCD3_V_PCD);			// depends on mem clock!!!
#endif
        LCCR4 = 0x00000000;
        // program FDADR0 (and FDADR1 if used)
        FDADR0 = (unsigned long)LCD_Descriptor_PAL;		//point to first descriptor
        FDADR1 = (unsigned long)LCD_Descriptor_LO;		//point to first descriptor

        //program LCCR0
        LCCR0 = 	(0   << LCD0_V_OUM)   |
                 (1   << LCD0_V_BM)    |
                 (0x0 << LCD0_V_PDD)   |
                 (0   << LCD0_V_QDM)   |
                 (0   << LCD0_V_DIS)   |
                 (0   << LCD0_V_DPD)   |
                 (1   << LCD0_V_PAS)   |
                 (1   << LCD0_V_EFM)   |
                 (1   << LCD0_V_IUM)   |
                 (1   << LCD0_V_SFM)   |
                 (1   << LCD0_V_LDM)   |
                 (0   << LCD0_V_SDS)   |
                 (0   << LCD0_V_CMS)   |
                 (1   << LCD0_V_ENB);

        Delay(5000);			// Delay (uSec) -- LCD panel requirment.
        LCCR0 |= (1 << LCD0_V_ENB);
#ifdef NEW_LCD

	GPDR3 |=  0x00100000;
	GPSR3  =  0x00100000;
        Delay(20000);
        GPCR3  =  0x00100000;/*a17400 change from zhou qiong */
#else

	GPDR3 |=  0x00100000;
	GPSR3  =  0x00100000;
#endif

        Delay(50000);			// Delay (uSec) -- LCD panel requirment.
}

/*****************************************************************************/


int putcToFB_8bit(char c, unsigned int row, unsigned int col)
{

        unsigned char *font;
        unsigned char *pix;
        unsigned int f;
        int r;

        if ((row > 15) || (col > 23)) {                                          /* check if row/col in range */
                return -1;
        };

        /* gen pointer into fb at upper left corner of char cell */
        if ((row * 18) < (LCD_Y-30)) {
                pix = &fb_8bit->pixel[((row*18)+1)][col*10];
        } else {
                row = 15;
                pix = &fb_8bit->pixel[((row*18)+1)][col*10];
        };

        font = &font9x16[((c & 0xff) << 4)];

        for (r=0; r<18; r++) {
                if ((r == 0) || (r == 17)) {
                        f = 0;
                } else {
                        f = *font++;
                };

                *pix++ = pal_bg;
                *pix++ = ((f>>7) & 0x1) ? (pal_fg) : pal_bg;
                *pix++ = ((f>>6) & 0x1) ? (pal_fg) : pal_bg;
                *pix++ = ((f>>5) & 0x1) ? (pal_fg) : pal_bg;
                *pix++ = ((f>>4) & 0x1) ? (pal_fg) : pal_bg;
                *pix++ = ((f>>3) & 0x1) ? (pal_fg) : pal_bg;
                *pix++ = ((f>>2) & 0x1) ? (pal_fg) : pal_bg;
                *pix++ = ((f>>1) & 0x1) ? (pal_fg) : pal_bg;
                *pix++ = ((f>>0) & 0x1) ? (pal_fg) : pal_bg;
                *pix++ = (pal_bg);
                pix += (LCD_X-10);
        }

        return 0;
}


/*****************************************************************************/


void printlcd(char *cntrl_string)
{

        int length, i;
        char c;

        c = 0;

        length = strlen(cntrl_string);

        for(i=0; i<length; i++) {
                c = cntrl_string[i];
                if(c == 0xa) {
                        if(lcd_row < 15) {
                                lcd_row++;
                                lcd_col = 0;
                                putcToFB_8bit(0,lcd_row,lcd_col);
                        } else {
                                lcd_row = 15;
                                scroll_line();
                                lcd_col = 0;
                                putcToFB_8bit(0,lcd_row,lcd_col);
                        }
                } else {
                        putcToFB_8bit(c,lcd_row,lcd_col);
                        lcd_col++;
                }
        }
        //cacheflush();
}


/*****************************************************************************/
//
// This is a simple function to draw either a HORIZONTAL or a VERTICAL line.
// If the line does not lie on the X or Y axis, this function returns -1
//

int drawline(int x0, int y0, int x1, int y1)
{
        int i, tempo;
        unsigned char *p;

        // test for the line not being horizontal or vertical
        if ((x0 != x1) && (y0 != y1)) {	
		return -1;
        }

        if (x0 == x1)	// this line is horizontal
        {
                if (y0 > y1) {			// make sure that the smaller value is in y0
                        tempo = y0;
                        y0 = y1;
                        y1 = tempo;
                }

                for (i=y0; i<=y1;) {		// col
                        p = &fb_8bit->pixel[x0][i];
                        *p = pal_fg;
                        i = i + 1;
                }

        } else {	// this line is vertical
		if (x0 > x1) {			// make sure that the smaller value is in x0
			tempo = x0;
                        x0 = x1;
                        x1 = tempo;
                }

                for (i=x0; i<=x1;) {   		// col
                        p = &fb_8bit->pixel[i][y0];
                        *p = pal_fg;
                        i = i + 1;
                }
        }
	
	return 1;
}

void Delay(int uSec)
{
        int delaytime;
        delaytime = (int)(uSec*3.6864);
        OSCR = 0;
        while( OSCR > 50)
                ;
        while((OSCR) < delaytime)
                ;
}

int checkkey()
{
        unsigned long data, temp;
        unsigned long timeout;

        GAFR0_L &= 0xFFFFFCFF;
        GPDR0 &= 0xFFFFFFEF;

        //a17400 debug, if no key down, directly return bad
        if ((GPLR0 & 0X10))
                return 0;
begin:
        SerialOutputString("begin scan gpio4\n ");
        //printlcd("begin scan gpio4\n ");

        timeout = 0x10000000;
        while(data=(GPLR0 & 0X10)) {
                if(--timeout==0)
                        goto bad;
        }
        Delay(1000);
        temp = GPLR0 & 0x10;
        if(data != temp)
                goto begin;

        SerialOutputString("key down\n ");
        //printlcd("key down\n ");

        timeout = 0x10000000;
        while(!(data=(GPLR0 & 0X10))) {
                if(--timeout==0)
                        goto bad;
        }
        SerialOutputString("key up\n ");
        //	printlcd("key up\n ");

        return 1;
bad:
        SerialOutputString("not detect gpio4\n ");
        //	printlcd("not detect gpio4\n ");
        return 0;
}

void Disablelcd()
{
        GPSR2 = 0x00004000;
        GPDR2 |= 0x00004000;
}
void Disablebklight()
{
        PWM_PWDUTY0 =0;
        GAFR0_U &= 0xfffffff0;
        GPDR0 &= 0xfffeffff;
}
